@@ -1,4 +1,4 @@
-#####################################################################
+###################################################################
 package Net::Amazon;
 ######################################################################
 # Mike Schilli <m@perlmeister.com>, 2003
@@ -8,25 +8,45 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION          = '0.21';
+our $VERSION          = '0.49';
+our $WSDL_DATE        = '2007-10-29';
+our $Locale           = 'us';
 our @CANNED_RESPONSES = ();
+our $IS_CANNED        = 0;
 
 use LWP::UserAgent;
 use HTTP::Request::Common;
 use XML::Simple;
 use Data::Dumper;
 use URI;
-use Log::Log4perl qw(:easy);
+use Log::Log4perl qw(:easy get_logger);
+use Time::HiRes qw(usleep gettimeofday tv_interval);
 
-use Net::Amazon::Request::ASIN;
-use Net::Amazon::Request::Artist;
-use Net::Amazon::Request::BrowseNode;
-use Net::Amazon::Request::Keyword;
-use Net::Amazon::Request::Wishlist;
-use Net::Amazon::Request::UPC;
-use Net::Amazon::Request::Similar;
-use Net::Amazon::Request::Power;
-use Net::Amazon::Request::TextStream;
+# Each key represents a search() type, and each value indicates which
+# Net::Amazon::Request:: class to use to handle it.
+use constant SEARCH_TYPE_CLASS_MAP => {
+    actor        => 'Actor',
+    artist       => 'Artist',
+    author       => 'Author',
+    asin         => 'ASIN',
+    blended      => 'Blended',
+    browsenode   => 'BrowseNode',
+    director     => 'Director',
+    ean          => 'EAN',
+    exchange     => 'Exchange',
+    isbn         => 'ISBN',
+    keyword      => 'Keyword',
+    manufacturer => 'Manufacturer',
+    musiclabel   => 'MusicLabel',
+    power        => 'Power',
+    publisher    => 'Publisher',
+    seller       => 'Seller',
+    similar      => 'Similar',
+    textstream   => 'TextStream',
+    title        => 'Title',
+    upc          => 'UPC',
+    wishlist     => 'Wishlist',
+};
 
 ##################################################
 sub new {
@@ -37,14 +57,22 @@ sub new {
         die "Mandatory paramter 'token' not defined";
     }
 
-    if(! exists $options{affiliate_id}) {
-        $options{affiliate_id} = "webservices-20";
-    }
-
     my $self = {
-        max_pages => 5,
+        strict         => 1,
+        response_dump  => 0,
+        rate_limit     => 1.0,  # 1 req/sec
+        max_pages      => 5,
+        ua             => LWP::UserAgent->new(),
         %options,
-               };
+    };
+
+    # XXX: this has to be set as soon as possible to ensure
+    # the validators pick up the correct locale.  I don't
+    # like the way this works, and need to think of a better
+    # solution.
+    if (exists $self->{locale}) {
+        $Net::Amazon::Locale = $self->{locale};
+    }
 
     help_xml_simple_choose_a_parser();
 
@@ -56,40 +84,21 @@ sub search {
 ##################################################
     my($self, %params) = @_;
 
-    my $req;
-
-    if(0) {
-    } elsif(exists $params{asin}) {
-        $req = Net::Amazon::Request::ASIN->new(%params);
-    } elsif(exists $params{artist}) {
-        $req = Net::Amazon::Request::Artist->new(%params);
-    } elsif(exists $params{blended}) {
-        $req = Net::Amazon::Request::Blended->new(%params);
-    } elsif(exists $params{wishlist}) {
-        $req = Net::Amazon::Request::Wishlist->new(
-                                   id => $params{wishlist}, %params);
-    } elsif(exists $params{upc}) {
-        $req = Net::Amazon::Request::UPC->new(%params);
-    } elsif(exists $params{keyword}) {
-        $req = Net::Amazon::Request::Keyword->new(%params);
-    } elsif(exists $params{similar}) {
-        $req = Net::Amazon::Request::Similar->new(asin => $params{similar},
-                                                  %params);
-    } elsif(exists $params{power}) {
-        $req = Net::Amazon::Request::Power->new(%params);
-    } elsif(exists $params{browsenode}) {
-        $req = Net::Amazon::Request::BrowseNode->new(%params);
-    } elsif(exists $params{manufacturer}) {
-        $req = Net::Amazon::Request::Manufacturer->new(%params);
-    } elsif(exists $params{textstream}) {
-        $req = Net::Amazon::Request::TextStream->new(%params);
-
-    } else {
-        warn "No Net::Amazon::Request type could be determined";
-        return;
+    foreach my $key ( keys %params ) {
+        next unless ( my $class = SEARCH_TYPE_CLASS_MAP->{$key} );
+        
+        return $self->_make_request($class, \%params);
     }
 
-    return $self->request($req);
+    # FIX?
+    # This seems like it really should be a die() instead...this is
+    # indicative of a programming problem. Generally speaking, it's
+    # best to issue warnings from a module--you can't be sure that the
+    # client has a stderr to begin with, or that he wants errors
+    # spewed to it.
+    warn "No Net::Amazon::Request type could be determined";
+
+    return undef;
 }
 
 ##################################################
@@ -97,25 +106,23 @@ sub intl_url {
 ##################################################
     my($self, $url) = @_;
 
-    # Every time Amazon is adding a new country to the web service,
-    # they're rolling a dice on what the new URL is going to be.
-    # This method will try to keep up with their crazy mappings.
-
     if(! exists $self->{locale}) {
         return $url;
     }
 
-    if ($self->{locale} eq "jp") {
-       $url =~ s/\.com/.co.jp/;
-       return $url;
+    if (0) {
+    } elsif ($self->{locale} eq "ca") {
+        $url =~ s/\.com/.ca/;
+    } elsif ($self->{locale} eq "de") {
+        $url =~ s/\.com/.de/;
+    } elsif ($self->{locale} eq "fr") {
+        $url =~ s/\.com/.fr/;
+    } elsif ($self->{locale} eq "jp") {
+        $url =~ s/\.com/.co.jp/;
+    } elsif ($self->{locale} eq "uk") {
+        $url =~ s/\.com/.co.uk/;
     }
 
-    if($self->{locale} eq "uk" or
-       $self->{locale} eq "de") {
-        $url =~ s/xml/xml-eu/;
-        return $url;
-    }
-        
     return $url;
 }
 
@@ -124,6 +131,7 @@ sub request {
 ##################################################
     my($self, $request) = @_;
 
+# XXX: Not sure if this bug exists in AWS4
     my $AMZN_WISHLIST_BUG_ENCOUNTERED = 0;
 
     my $resp_class = $request->response_class();
@@ -134,21 +142,26 @@ sub request {
     my $res  = $resp_class->new();
 
     my $url  = URI->new($self->intl_url($request->amzn_xml_url()));
-    my $page = $request->{page};
+    my $page = $request->page();
     my $ref;
 
-    {
-        my %params = $request->params();
-        $params{page}   = $page;
+    REQUEST: {
+        my %params = $request->params(page => $page);
         $params{locale} = $self->{locale} if exists $self->{locale};
 
         $url->query_form(
-            'dev-t' => $self->{token},
-            't'     => $self->{affiliate_id},
-            %params,
+            'Service'        => 'AWSECommerceService',
+            'AWSAccessKeyId' => $self->{token},
+            'Version'        => $WSDL_DATE,
+            map { $_, $params{$_} } sort keys %params,
         );
 
+        DEBUG(sub { "request: params = " . Dumper(\%params) . "\n"});
+
         my $urlstr = $url->as_string;
+
+        DEBUG(sub { "urlstr=" . $urlstr });
+
         my $xml = fetch_url($self, $urlstr, $res);
 
         if(!defined $xml) {
@@ -157,8 +170,8 @@ sub request {
 
         DEBUG(sub { "Received [ " . $xml . "]" });
 
-        my $xs = XML::Simple->new();
-        $ref = $xs->XMLin($xml);
+        # Let the response class parse the XML
+        $ref = $res->xml_parse($xml);
 
         # DEBUG(sub { Data::Dumper::Dumper($ref) });
 
@@ -169,62 +182,38 @@ sub request {
             return $res;
         }
 
-        if(exists $ref->{TotalPages}) {
-            INFO("Page $page/$ref->{TotalPages}");
-        }
-
-        if(exists $ref->{ErrorMsg}) {
-
-            if($AMZN_WISHLIST_BUG_ENCOUNTERED &&
-               $ref->{ErrorMsg} =~ /no exact matches/) {
-                DEBUG("End of buggy wishlist detected");
-                last;
-            }
-                
-	    if (ref($ref->{ErrorMsg}) eq "ARRAY") {
-	      # multiple errors, set arrary ref
-	      $res->messages( $ref->{ErrorMsg} );
-	    } else {
-	      # single error, create array
-	      $res->messages( [ $ref->{ErrorMsg} ] );
-            }
-            ERROR("Fetch Error: " . $res->message );
-            $res->status("");
+        $res->current_page($ref, $page);
+        $res->set_total_results($ref);
+        
+        my $rc = $res->is_page_error($ref);
+        if ($rc == 0) {
             return $res;
+        } elsif ($rc == -1) {
+            last;
         }
 
         my $new_items = $res->xmlref_add($ref);
+
         DEBUG("Received valid XML ($new_items items)");
 
         # Stop if we've fetched max_pages already
-        if($self->{max_pages} <= $page) {
+        if(defined $page && $self->{max_pages} <= $page) {
             DEBUG("Fetched max_pages ($self->{max_pages}) -- stopping");
             last;
         }
 
-        # Work around the Amazon bug not setting TotalPages properly
-        # for wishlists
-        if(ref($res) =~ /Wishlist/ and
-           !exists $ref->{TotalPages}  and
-           $new_items == 10
-          ) {
-            $AMZN_WISHLIST_BUG_ENCOUNTERED = 1;
-            DEBUG("Trying to fetch additional wishlist page (AMZN bug)");
+        if($res->is_page_available($ref, $new_items, $page)) {
             $page++;
-            redo;
-        }
-
-        if(exists $ref->{TotalPages} and
-           $ref->{TotalPages} > $page) {
-            DEBUG("Page $page of $ref->{TotalPages} fetched - continuing");
-            $page++;
-            redo;
+            redo REQUEST;
         }
 
         # We're gonna fall out of this loop here.
     }
 
     $res->status(1);
+    # We have a valid response, so if TotalResults isn't set, 
+    # we most likely have a single response
+    $res->total_results(1) unless defined $res->total_results();
     return $res;
 }
 
@@ -238,6 +227,7 @@ sub fetch_url {
     INFO("Fetching $url");
 
     if(@CANNED_RESPONSES) {
+        $IS_CANNED = 1;
         INFO("Serving canned response (testing)");
         return shift @CANNED_RESPONSES;
     }
@@ -252,23 +242,53 @@ sub fetch_url {
         INFO("Cache miss");
     }
 
-    my $ua = LWP::UserAgent->new();
+    my $ua = $self->{ua};
+    $ua->env_proxy();
+
     my $resp;
 
     {
+        # wait up to a second before the next request so
+        # as to not violate Amazon's 1 query per second
+        # rule (or the configured rate_limit).
+        $self->pause() if $self->{strict};
+
         $resp = $ua->request(GET $url);
 
+        $self->reset_timer() if $self->{strict};
+
         if($resp->is_error) {
-            $res->status("");
-            $res->messages( [ $resp->message ] );
-            return undef;
+            # retry on 503 Service Unavailable errors
+            if ($resp->code == 503) {
+                if ($max_retries-- >= 0) {
+                    INFO("Temporary Amazon error 503, retrying");
+                    redo;
+                } else {
+                    INFO("Out of retries, giving up");
+                    $res->status("");
+                    $res->messages( [ "Too many temporary Amazon errors" ] );
+                    return undef;
+                }
+            } else {
+                $res->status("");
+                $res->messages( [ $resp->message ] );
+                return undef;
+            }
         }
 
-        if($resp->content =~ /<ErrorMsg>/ &&
+        if($self->{response_dump}) {
+            my $dumpfile = "response-$self->{response_dump}.txt";
+            open FILE, ">$dumpfile" or die "Cannot open $dumpfile";
+            print FILE $resp->content();
+            close FILE;
+            $self->{response_dump}++;
+        }
+
+        if($resp->content =~ /<Errors>/ &&
+            # Is this the same value of AWS4?
            $resp->content =~ /Please retry/i) {
             if($max_retries-- >= 0) {
                 INFO("Temporary Amazon error, retrying");
-                sleep(1);
                 redo;
             } else {
                 INFO("Out of retries, giving up");
@@ -314,6 +334,33 @@ EOT
     }
 }
 
+# An accessor for backward compatability with AWS3.
+##################################################
+sub make_compatible_accessor{
+##################################################
+    my($package, $old_name, $new_name) = @_;
+
+    no strict qw(refs);
+
+    my $code = <<EOT;
+        *{"$package\\::$old_name"} = sub {
+            my(\$self, \$value) = \@_;
+
+            if(defined \$value) {
+                \$self->{$new_name} = \$value;
+            }
+            if(exists \$self->{$new_name}) {
+                return (\$self->{$new_name});
+            } else {
+                return "";
+            }
+        }
+EOT
+    if(! defined *{"$package\::$old_name"}) {
+        eval $code or die "$@";
+    }
+}
+
 ##################################################
 # Make accessors for arrays
 ##################################################
@@ -334,7 +381,12 @@ sub make_array_accessor {
                 }
             }
                # Return a list
-            return \@{\$self->{$name}};
+            if(exists \$self->{$name} and
+               ref \$self->{$name} eq "ARRAY") {
+                return \@{\$self->{$name}};
+            }
+
+            return undef;
         }
 EOT
 
@@ -343,6 +395,27 @@ EOT
     }
 }
 
+##################################################
+sub walk_hash_ref {
+##################################################
+    my ($package, $href, $aref) = @_;
+
+    return $href if scalar(@$aref) == 0;
+
+    my @a;
+    push @a, $_ for @$aref;
+
+    my $tail = pop @a;
+    my $ref = $href;
+
+    for my $part (@a) {
+        $ref = $ref->{$part};
+    }
+    
+    return $ref->{$tail};
+}
+
+
 ##################################################
 sub artist {
 ##################################################
@@ -352,6 +425,70 @@ sub artist {
     return ($self->artists($nameref))[0];
 }
 
+##################################################
+sub version {
+##################################################
+    my($self) = @_;
+    return $self->{Version};
+}
+
+##################################################
+sub current_page {
+##################################################
+    my($self, $ref, $page) = @_;
+    if(exists $ref->{Items}->{TotalPages}) {
+        INFO("Page $page/$ref->{Items}->{TotalPages}");
+    }
+}
+
+##################################################
+sub set_total_results {
+##################################################
+    my($self, $ref) = @_;
+    if(exists $ref->{Items}->{TotalResults}) {
+        $self->total_results( $ref->{Items}->{TotalResults} );
+    }
+}
+
+##################################################
+sub is_page_error {
+##################################################
+    my($self, $ref) = @_;
+
+    if(exists $ref->{Items}->{Request}->{Errors}) {
+        my $errref = $ref->{Items}->{Request}->{Errors};
+
+        if (ref($errref->{Error}) eq "ARRAY") {
+            my @errors;
+            for my $e (@{$errref->{Error}}) {
+                push @errors, $e->{Message};
+            }
+            # multiple errors, set arrary ref
+            $self->messages( @errors );
+        } else {
+            # single error, create array
+            $self->messages( [ $errref->{Error}->{Message} ] );
+        }
+
+        ERROR("Fetch Error: " . $self->message );
+        $self->status("");
+        return 0;
+    }
+    return 1;
+}
+
+##################################################
+sub is_page_available {
+##################################################
+    my($self, $ref, $new_items, $page) = @_;
+    if(exists $ref->{Items}->{TotalPages} and
+              $ref->{Items}->{TotalPages} > $page and 
+              $IS_CANNED ne 1) {
+        DEBUG("Page $page of $ref->{Items}->{TotalPages} fetched - continuing");
+        return 1;
+    }
+    return 0;
+}
 
 ##################################################
 sub xmlref_add {
@@ -359,6 +496,7 @@ sub xmlref_add {
     my($self, $xmlref) = @_;
 
     my $nof_items_added = 0;
+    return $nof_items_added unless defined $xmlref;
 
     # Push a nested hash structure, retrieved via XMLSimple, onto the
     # object's internal 'xmlref' entry, which holds a ref to an array, 
@@ -368,21 +506,21 @@ sub xmlref_add {
     #DEBUG("xmlref_add ", Data::Dumper::Dumper($xmlref));
 
     unless(ref($self->{xmlref}) eq "HASH" &&
-           ref($self->{xmlref}->{Details}) eq "ARRAY") {
-        $self->{xmlref}->{Details} = [];
+           ref($self->{xmlref}->{Items}) eq "ARRAY") {
+        $self->{xmlref}->{Items} = [];
     }
 
-    if(ref($xmlref->{Details}) eq "ARRAY") {
-        # Is it an array of items?
-        push @{$self->{xmlref}->{Details}}, @{$xmlref->{Details}};
-        $nof_items_added = scalar @{$xmlref->{Details}};
+    if(ref($xmlref->{Items}->{Item}) eq "ARRAY") {
+        push @{$self->{xmlref}->{Items}}, @{$xmlref->{Items}->{Item}};
+        $nof_items_added = scalar @{$xmlref->{Items}->{Item}};
     } else {
-        # It is a single item
-        push @{$self->{xmlref}->{Details}}, $xmlref->{Details};
-        $nof_items_added = 1;
+        if (exists $xmlref->{Items}->{Item}->{ItemAttributes}) {
+            push @{$self->{xmlref}->{Items}}, $xmlref->{Items}->{Item};
+            $nof_items_added = 1;
+        }
     }
 
-    #DEBUG("xmlref_add (after):", Data::Dumper::Dumper($self));
+    DEBUG("xmlref_add (after):", Data::Dumper::Dumper($self));
     return $nof_items_added;
 }
 
@@ -403,13 +541,70 @@ sub help_xml_simple_choose_a_parser {
     }
 }
 
+##################################################
+# This timer makes sure we don't query Amazon more
+# than once a second.
+##################################################
+sub reset_timer {
+##################################################
+
+    my $self = shift;
+    $self->{t0} = [gettimeofday];
+}
+
+##################################################
+# Pause for up to a second if necessary.
+##################################################
+sub pause {
+##################################################
+
+    my $self = shift;
+    return unless ($self->{t0});
+
+    my $t1 = [gettimeofday];
+    my $dur = (1.0/$self->{rate_limit} - 
+               tv_interval($self->{t0}, $t1)) * 1000000;
+    if($dur > 0) {
+            # Use a pseudo subclass for the logger, since the app
+            # might not want to log that as 'ERROR'. Log4perl's
+            # inheritance mechanism makes sure it does the right
+            # thing for the current class.
+        my $logger = get_logger(__PACKAGE__ . "::RateLimit");
+        $logger->error("Ratelimiting: Sleeping $dur microseconds"); 
+        usleep($dur);
+    }
+}
+
+##
+## 'PRIVATE' METHODS
+##
+
+# $self->_make_request( TYPE, PARAMS )
+#
+# Takes a TYPE that corresponds to a Net::Amazon::Request
+# class, require()s that class, instantiates it, and returns
+# the result of that instance's request() method.
+#
+sub _make_request {
+    my ($self, $type, $params) = @_;
+
+    my $class = "Net::Amazon::Request::$type";
+
+	# XXX: change me back, this makes debugging a little difficult.
+    eval "require $class";
+
+    my $req = $class->new(%{$params});
+    
+    return $self->request($req);
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-Net::Amazon - Framework for accessing amazon.com via SOAP and XML/HTTP
+Net::Amazon - Framework for accessing amazon.com via REST
 
 =head1 SYNOPSIS
 
@@ -429,7 +624,7 @@ Net::Amazon - Framework for accessing amazon.com via SOAP and XML/HTTP
 =head1 ABSTRACT
 
   Net::Amazon provides an object-oriented interface to amazon.com's
-  SOAP and XML/HTTP interfaces. This way it's possible to create applications
+  REST interface. This way it's possible to create applications
   using Amazon's vast amount of data via a functional interface, without
   having to worry about the underlying communication mechanism.
 
@@ -446,7 +641,9 @@ like
 which you pass your personal amazon developer's token (can be obtained
 from L<http://amazon.com/soap>) and (optionally) the maximum number of 
 result pages the agent is going to request from Amazon in case all
-results don't fit on a single page (typically holding 20 items).
+results don't fit on a single page (typically holding 20 items).  Note that
+each new page requires a minimum delay of 1 second to comply with Amazon's
+one-query-per-second policy.
 
 According to the different search methods on Amazon, there's a bunch
 of different request types in C<Net::Amazon>. The user agent's 
@@ -458,19 +655,40 @@ depending on which parameters you pass to it:
 =item C<< $ua->search(asin => "0201360683") >>
 
 The C<asin> parameter has Net::Amazon search for an item with the 
-specified ASIN. Returns at most one result.
+specified ASIN. If the specified value is an arrayref instead of a single
+scalar, like in
+
+    $ua->search(asin => ["0201360683", "0596005083"]) 
+
+then a search for multiple ASINs is performed, returning a list of 
+results.
+
+=item C<< $ua->search(actor => "Adam Sandler") >>
+
+The C<actor> parameter has the user agent search for items created by
+the specified actor. Can return many results.
 
 =item C<< $ua->search(artist => "Rolling Stones") >>
 
 The C<artist> parameter has the user agent search for items created by
 the specified artist. Can return many results.
 
+=item C<< $ua->search(author => "Robert Jordan") >>
+
+The C<author> parameter has the user agent search for items created by
+the specified author. Can return many results.
+
 =item C<< $ua->search(browsenode=>"4025", mode=>"books" [, keywords=>"perl"]) >>
 
 Returns a list of items by category ID (node). For example node "4025"
 is the CGI books category.  You can add a keywords parameter to filter 
 the results by that keyword.
 
+=item C<< $ua->search(exchange => 'Y04Y3424291Y2398445') >>
+
+Returns an item offered by a third-party seller. The item is referenced
+by the so-called I<exchange ID>.
+
 =item C<< $ua->search(keyword => "perl xml", mode => "books") >>
 
 Search by keyword, mandatory parameters C<keyword> and C<mode>.
@@ -486,6 +704,12 @@ Can return many results.
 Music search by UPC (product barcode), mandatory parameter C<upc>.
 C<mode> has to be set to C<music>. Returns at most one result.
 
+=item C<< $ua->search(isbn => "0439784549") >>
+
+Book search by ISBN (International Standard Book Number), mandatory parameter
+C<isbn>.  Returns at most one result.  When searching non-US locales use the
+13-digit ISBN.
+
 =item C<< $ua->search(similar => "0201360683") >>
 
 Search for all items similar to the one represented by the ASIN provided.
@@ -496,12 +720,36 @@ Can return many results.
 Initiate a power search for all books matching the power query.
 Can return many results. See L<Net::Amazon::Request::Power> for details.
 
-=item C<< $ua->search(manufacturer => "o'reilly", mode => "books") >>
+=item C<< $ua->search(manufacturer => "Disney") >>
 
 Initiate a search for all items made by a given manufacturrer.
 Can return many results. See L<Net::Amazon::Request::Manufacturer> 
 for details.
 
+=item C<< $ua->search(musiclabel => "Arista") >>
+
+Initiate a search for all items made by a given music label. Can return many
+results. See Net::Amazon::Request::MusicLabel for details.
+
+
+=item C<< $ua->search(publisher => "o'reilly") >>
+
+Initiate a search for all items made by a given publisher. Can return many
+results. See Net::Amazon::Request::Publisher for details.
+
+=item C<< $ua->search(blended => "Perl") >>
+
+Initiate a search for items in all categories.
+
+=item C<< $ua->search(seller => "A2GXAGU54VOP7") >>
+
+Start a search on items sold by a specific third-party seller, referenced
+by its ID (not seller name).
+
+=item C<< $ua->search(textstream => "Blah blah Rolling Stones blah blah") >>
+
+Find items related to keywords within a text stream.
+
 =back
 
 The user agent's C<search> method returns a response object, which can be 
@@ -513,7 +761,8 @@ checked for success or failure:
       print "Error: ", $resp->message(), "\n";
   }
 
-In case the request succeeds, the response contains one or more
+In case the request for an item search 
+succeeds, the response contains one or more
 Amazon 'properties', as it calls the products found.
 All matches can be retrieved from the Response 
 object using it's C<properties()> method.
@@ -528,9 +777,12 @@ Response objects always have the methods
 C<is_success()>,
 C<is_error()>,
 C<message()>,
+C<total_results()>,
 C<as_string()> and
 C<properties()> available.
 
+C<total_results()> returns the total number of results the search
+yielded.
 C<properties()> returns one or more C<Net::Amazon::Property> objects of type
 C<Net::Amazon::Property> (or one of its subclasses like
 C<Net::Amazon::Property::Book>, C<Net::Amazon::Property::Music>
@@ -543,7 +795,9 @@ in Amazon's database:
              $_->OurPrice(), "\n";
     }
 
-Commonly available accessors are 
+In scalar context, C<properties()> just returns the I<first> 
+C<Net::Amazon::Property> object found.
+Commonly available accessors to C<Net::Amazon::Property> objects are
 C<OurPrice()>,
 C<ImageUrlLarge()>,
 C<ImageUrlMedium()>,
@@ -556,7 +810,15 @@ C<Manufacturer()>,
 C<UsedPrice()>,
 C<ListPrice()>,
 C<ProductName()>,
-C<Availability()>.
+C<Availability()>,
+C<SalesRank()>,
+C<CollectiblePrice()>,
+C<CollectibleCount()>,
+C<NumberOfOfferings()>,
+C<UsedCount()>,
+C<ThirdPartyNewPrice()>,
+C<ThirdPartyNewCount()>,
+C<similar_asins()>.
 For details, check L<Net::Amazon::Property>.
 
 Also, the specialized classes C<Net::Amazon::Property::Book> and
@@ -587,11 +849,19 @@ request objects:
 Search by ASIN, mandatory parameter C<asin>. 
 Returns at most one result.
 
+=item Net::Amazon::Request::Actor
+
+Music search by Actor, mandatory parameter "actor". Can return many results.
+
 =item Net::Amazon::Request::Artist
 
 Music search by Artist, mandatory parameter C<artist>.
 Can return many results.
 
+=item Net::Amazon::Request::Author
+
+Music search by Author, mandatory parameter "author". Can return many results.
+
 =item Net::Amazon::Request::BrowseNode
 
 Returns category (node) listing. Mandatory parameters C<browsenode>
@@ -622,7 +892,40 @@ for details. Mandatory parameter C<power>.
 =item Net::Amazon::Request::Manufacturer
 
 Searches for all items made by a given manufacturer. Mandatory parameter
-C<manufacturer>.
+C<manufacturer>.  With the change to AWS4, manufacturer is no longer used to
+search for publishers.  To search via publisher use
+Net::Amazon::Request::Publisher.
+
+=item Net::Amazon::Request::Publisher
+
+Searches for all items made by a given manufacturer. Mandatory
+parameter C<publisher>.
+
+=item Net::Amazon::Request::Similar
+
+Finds items similar to a given one.
+
+=item Net::Amazon::Request::Wishlist
+
+Find item on someone's wish list.
+
+=item Net::Amazon::Request::Seller
+
+Searches for a third-party seller on Amazon by seller ID. This search
+is different than the previous ones, since it doesn't return Amazon
+items, but a single seller record. Don't use the C<properties()> method
+on the response, use C<result()> instead, which returns a 
+L<Net::Amazon::Result::Seller> object. Check the manpage for details.
+
+=item Net::Amazon::Request::Exchange
+
+Searches for items offered by third-party sellers. Items are referenced
+by their so-called I<Exchange ID>.
+Similar to L<Net::Amazon::Request::Seller>,
+this request doesn't return a list of Amazon properties, so please use
+C<result()> instead, which will return a I<single>
+L<Net::Amazon::Result::Seller::Listing> item.
+Check the manpage for details on what attributes are available there.
 
 =back
 
@@ -632,7 +935,7 @@ as an example):
 
     my $req = Net::Amazon::Request::Keyword->new(
         keyword   => 'perl',
-        mode      => 'books',
+	mode      => 'books',
     );
 
 and are handed over to the user agent like that:
@@ -658,17 +961,26 @@ Additional optional parameters:
 
 =item C<< max_pages => $max_pages >>
 
-sets how many 
+Sets how many 
 result pages the module is supposed to fetch back from Amazon, which
-only sends back 10 results per page. 
+only sends back 10 results per page.  
+Since each page requires a new query to Amazon, at most one query 
+per second will be made in C<strict> mode to comply with Amazon's terms 
+of service. This will impact performance if you perform a search 
+returning many pages of results.
 
-=item C<< affiliate_id => $affiliate_id >>
+=item C<< strict => 1 >>
 
-your Amazon affiliate ID, if you have one. It defaults to 
-C<webservices-20> which is currently (as of 06/2003) 
-required by Amazon.
+Makes sure that C<Net::Amazon> complies with Amazon's terms of service
+by limiting the number of outgoing requests to 1 per second. Defaults
+to C<1>, enabling rate limiting as defined via C<rate_limit>.
 
-=back
+=item C<< rate_limit => $reqs_per_sec >>
+
+Sets the rate limit to C<$reqs_per_sec> requests per second if 
+rate limiting has been enabled with C<strict> (see above).
+Defaults to C<1>, limiting the number of outgoing requests to 
+1 per second.
 
 =item C<< $resp = $ua->request($request) >>
 
@@ -678,14 +990,36 @@ corresponding C<Net::Amazon::Response::*> type.
 
 =back
 
+=head2 Modes
+
+Every search method takes a mode parameter.  The mode parameter is used to
+narrow the search to a specific field.  For example, when searching by actor
+you can search by DVD, DigitalMusic, Merchants, VHS, and Video.  By default DVD
+is used when searching by actor.  The modes available are dependent upon the
+type of search, and locale the search is conducted in.
+
+Determining the modes available to a search type are auto-generated from data
+published by Amazon on their web site.  A man page is available for each type
+of search.  The man page lists the default value if a mode is not specified.  A
+list of mode values is also provided.  The man page's name is of the form
+Net::Amazon::Validate::ItemSearch::E<lt>localeE<gt>::E<lt>typeE<gt>.
+
+E<lt>localeE<gt> is one of any ca, de, fr, jp, uk, or us.
+
+E<lt>typeE<gt> is one of Actor, Artist, Author, BrowseNode, Director, Keywords,
+Manufacturer, MusicLabel, Power, Publisher, TextStream, or UPC.
+
 =head2 Accessing foreign Amazon Catalogs
 
-As of this writing (07/2003), Amazon also offers its web service for
-the UK, Germany, and Japan. Just pass in
+As of this writing (01/2007), Amazon also offers its web service for the UK,
+Germany, Canada, France, and Japan. Just pass in
 
-    locale => 'uk'
+    locale => 'ca'
     locale => 'de'
+    locale => 'fr'
     locale => 'jp'
+    locale => 'uk'
+    locale => 'us'
 
 respectively to C<Net::Amazon>'s constructor C<new()> and instead of returning
 results sent by the US mothership, it will query the particular country's
@@ -744,6 +1078,10 @@ And here's one displaying someone's wishlist:
         print $resp->message(), "\n";
     }
 
+DETAILS
+        Net::Amazon is based on Amazon Web Services version 4, and uses
+        WSDL version 2007-01-17.
+
 =head1 CACHING
 
 Responses returned by Amazon's web service can be cached locally.
@@ -782,6 +1120,35 @@ adheres to the following interface can be used:
         # Return a cached value, 'undef' if it doesn't exist
     $cache->get($key);
 
+=head1 PROXY SETTINGS
+
+C<Net::Amazon> uses C<LWP::UserAgent> under the hood to send
+web requests to Amazon's web site. If you're in an environment where
+all Web traffic goes through a proxy, there's two ways to configure that.
+
+First, C<Net::Amazon> picks up proxy settings from environment variables:
+
+    export http_proxy=http://proxy.my.place:8080
+
+in the surrounding shell or setting
+
+    $ENV{http_proxy} = "http://proxy.my.place:8080";
+
+in your Perl script
+will route all requests through the specified proxy.
+
+Secondly, you can
+pass a user agent instance to Net::Amazon's constructor:
+
+    use Net::Amazon;
+    use LWP::UserAgent;
+
+    my $ua = LWP::UserAgent->new();
+    my $na = Net::Amazon->new(ua => $ua, token => 'YOUR_AMZN_TOKEN');
+    # ...
+
+This way, you can configure C<$ua> up front before Net::Amazon will use it.
+
 =head1 DEBUGGING
 
 If something's going wrong and you want more verbosity, just bump up
@@ -952,11 +1319,11 @@ Apply your changes to this development tree.
 Run a diff between the tree and your changes it in this way:
 
     cd Net-Amazon
-    cvs diff -Nau >patch_to_mike.txt
+    cvs diff -Nau >patch_to_christopher.txt
 
 =item *
 
-Email me C<patch_to_mike.txt>. If your patch works (and you've included
+Email me C<patch_to_christopher.txt>. If your patch works (and you've included
 test cases and documentation), I'll apply it on the spot.
 
 =back
@@ -1018,20 +1385,29 @@ list net-amazon-devel@lists.sourceforge.net
 
 Mike Schilli, E<lt>na@perlmeister.comE<gt> (Please contact me via the mailing list: net-amazon-devel@lists.sourceforge.net )
 
+Maintainers: Christopher Boumenot, E<lt>boumenot+na@gmail.comE<gt>
+
 Contributors (thanks y'all!):
 
+    Andy Grundman <andy@hybridized.org>
     Barnaby Claydon <bclaydon@perseus.com>
+    Batara Kesuma <bkesuma@gaijinweb.com>
     Bill Fitzpatrick
+    Brian <brianbrian@gmail.com>
     Brian Hirt <bhirt@mobygames.com>
+    Dan Kreft <dan@kreft.net>
     Dan Sully <daniel@electricrain.com>
     Jackie Hamilton <kira@cgi101.com>
     Konstantin Gredeskoul <kig@get.topica.com>
+    Lance Cleveland <lancec@proactivewm.com>
     Martha Greenberg <marthag@mit.edu>
     Martin Streicher <martin.streicher@apress.com>
     Mike Evron <evronm@dtcinc.net>
     Padraic Renaghan <padraic@renaghan.com>
     rayg <rayg@varchars.com>
     Robert Graff <rgraff@workingdemo.com>
+    Robert Rothenberg <wlkngowl@i-2000.com>
+    Steve Rushe <steve@deeden.co.uk>
     Tatsuhiko Miyagawa <miyagawa@livedoor.jp>
     Tony Bowden <tony@kasei.com>
 
